pro mk_ssc,  week,  outdir=outdir, loud=loud, $
  noscatter=noscatter, scatter=scatter, nodeka=nodeka, nokleen=nokleen, $
  vignette=vignette, novignette=novignette, $
  nodestreak=nodestreak, destreak=destreak, $
  noclean=noclean, noleak=noleak, no_second_order_leak=no_second_order_leak, $
  prefix=prefix,  ss=ss, triplet=triplet, nonormalize=nonormalize, $
  saa_state=saa_state, ifiles=ifiles, outfile=outfile, $
  subdir=subdir, yyyy=yyyy, filename_only=filename_only, _extra=_extra, $
  uncert=uncert, synsfc=synsfc, debug=debug


;+
;   Name: mk_ssc
;
;   Purpose: make SXT "Final" science composite movie - de-spagetti mk_sfd
;
;   Input Parameters:
;      week - the week to process 
; 
;   Output Parameters:
;     NONE
;
;   Keyword Parameters:
;      nodeka     - dont deka normalize
;      novignette - dont vignette correct
;      noclean    - dont call sxt_clean (despike and optional)
;      noscatter  - dont call sxt_scatter
;      triplet    - make 3-image composite SSTs
;      prefix - output file prefix (default='ssc' (pair), 'sst' implies triple 
;      ifiles - explicit filelist (in lieu of WEEK string)
;      outdir - output directory
;      subdir - optional subdirectory 
;      filename_only - if set, just derive planned filename and return in OUTFILE
;      ss     - Set ss to limit the number of composites made to ss (saves time)
;      debug  - Stop before return for debugging
;  
;   Side Effects:
;      generates an 'ssc' file - floating point, normalized to counts/sec/pix
;   
;   History:
;      14-Sep-1998 - S.L.Freeland  - Written (YPOP/MSU)
;      16-Sep-1998 - Various mods - include proc flags in file name.
;      17-Sep-1998 - Add scattered light correction and /NOSCATTER
;      18-Dec-1998 - Change normalization-> from deka-norm->counts/sec/pixel
;                    Change default prefix to SSC (Sxt Science Composite)
;                    Triplet->                SST 
;      21-Mar-1999 - S.L.Freeland - change /sxt_kleen->sxt_clean
;                    Add /destreak (sxt_xsaa)hook/logic
;      21-apr-1999 - S.L.Freeland - allow filelist in lieu of week string
;      12-aug-1999 - S.L.Freeland - add /YYYY,  /YYYYMM, and /YYYYMMDD keywords
;                    (subdirectory formats)
;      29-Sep-1999 - S.L.Freeland - add /FILENAME_ONLY keyword and function
;      26-Oct-1999 - S.L.Freeland - add /NOLEAK ( -> sxt_prep )
;      18-Jan-2001 - S.L.Freeland - add /UNCERT
;       8-May-2001 - S.L.Freeland - add /SYNSFC and flag (appended)
;      11-Dec-2001 - D.McKenzie - revised handling of uncertainties to
;		     work for composites, including normalization and
;		     output.  My hacks are on lines 197, 232, 241, 248, 264.
;      23-jul-2003 - S.L.Freeland - add statistical uncertainties
;                                   via sxt_dn_unc per NN, 3MK default 
;       7-Nov-2003 - S.L.Freeland - set MAXDT=600 in call to mk_sff_pair
;                                   (maximum permitted dT pair seperation)
;      10-nov-2003 - S.L.Freeland - add SYNSFC to box-message output
;                                   and optionally look at environmental 
;                                   (fixes file name... )   
;      13-nov-2003 - S.L.Freeland - change MAXDT from 600 -> 660
;      19-nov-2003 - S.L.Freeland - move 2nd order leak correction to 
;                                   this routine following compositing
;                                   step (don't pass keyword -> sxt_prep) 
;      18-dec-2003 - S.L.Freeland - change mk_sff_pair call 
;                                   for two image case 
;	2-jan-2004 - LWA - Restored 2nd order leak in sxt_prep call, line 234.
;	7-jan-2004 - LWA - Edited boxmessage to handle triplets.
;       8-jan-2004 - S.L.Freeland - avoid sxt_deleak call if pre-1st 
;                                   entrance filter failure
;	9-jan-2004 - LWA - Restricted McKenzie boxmessages to loud=1.
;			 - Cleaned up code, removed all ^M from DOS edits.
;			 - Put sxt_deleak step back into sxt_prep.  
;			 - Commented out Sam's changes of 19-nov-2003.
;      10-Jan-2004 - LWA - Deleted obsolete keyword refresh.
;			 - Removed forced uncert processing.
;      16-jan-2004 - S.L.Freeland - sxt_scatter2 -> sxt_decon updates merged 
;      19-Jan-2004 - LWA - Inserted test for images with severe CCD bleed.
;      20-Jan-2004 - LWA - Now reading xdata with "roadmap" for use in
;			mk_sff_pairs.  Include in common block.
;			   Moved bleed test to mk_sff_pair.pro.
;      24-Jan-2004 - LWA - Now doing all image selection tasks , including
;			sxt-satscreen, within mk_sff_pair.pro.
;      27-Jan-2004 - LWA - Debugs and fiddles.
;      20-aug-2004 - S.L.Freeland - set normalization .HIS algorithm flag
;       3-Sep-2004 - LWA - Moved sxt_deleak (2nd order leak) to final step.
;-
common mk_ssc_blk1, lweek, files, roadmap, xdata

mk_ssc_version='.1'  
loud=keyword_set(loud)
noleak=keyword_set(noleak)
triplet=keyword_set(triplet)
debug=keyword_set(debug)

if keyword_set(loud) then begin
   box_message,'McKenzie''s hack of MK_SSC'   ; To allow output of uncert, 11-dec-01
   box_message,'FORCING UNCERT PROCESSING'
endif   

; My hacks are on lines 197, 232, 241, 248, 264.  (McKenzie)

; Processing Flags (boolean in file name of form xxxx (0=>off,1=>on)
; ORDER>> SCATTER | VIGNETTE | CLEAN | DESTREAK | NORMALIZE | 2ndLEAK | synsfc

scatter=keyword_set(scatter)               ; Default OFF
vignette=keyword_set(vignette)             ; Default OFF
destreak=keyword_set(destreak)             ; Default OFF
soleak=1-keyword_set(no_second_order_leak) ; Default ON
clean=(1-keyword_set(noclean)) or destreak ; Default ON
synsfc=(keyword_set(synsfc)) or $
   get_logenv('sxt_syn_sfc') eq '1'            ; Default OFF
help,synsfc
deka=0                                     ; S.L.F 18-dec-1998 Default Changed
normalize=1-keyword_set(nonormalize)       ; Default ON

;------- make the processing flags part of the file name ------------
flagstring=strcompress(string(scatter)+ $
  string(vignette)+string(clean)+string(destreak)+string(normalize) + $
  string(soleak) + string(synsfc),/remove)

; ------------ define output directory ----------------
defdir=get_logenv('DIR_SXT_SSC')
case 1 of
  data_chk(outdir,/string,/scalar):                ; user supplied
  file_exist(defdir):              outdir =defdir  ; online
  else: outdir=curdir()                            ; current directory
endcase

if not keyword_set(outdir) then outdir=curdir()
case 1 of
   data_chk(prefix,/string,/scalar):                ; user supplied
   keyword_set(triplet):             prefix='sst'   ; triplet movie
   else:                             prefix='ssc'   ; default (pair) movie
endcase   

case 1 of
   keyword_set(saa_state): prefix=prefix + 'saa' ; saa only
   n_elements(saa_state) eq 1: prefix=prefix + 'nosaa'
   else:
endcase   

loud=keyword_set(loud)

if n_elements(lweek) eq 0 then lweek=''

; -------- read roadmaps ------------

case 1 of
   data_chk(week,/string): begin
     box_message,'Checking CDROM for associated week files...'
     files=cdrom_files(week=week,/sfr) 
     froot=week
   endcase
   data_chk(ifiles,/string): begin
      box_message,'Using user supplied filelist...'     
      files=ifiles
      froot=time2file(file2time(ifiles(0))) 
      case 1 of
         keyword_set(yyyy): subdir=strmid(froot,0,4)     ; prepend ../YYYY/...
	 data_chk(subdir,/string): sub=subdir
         else: subdir=''
      endcase
      outdir=concat_dir(outdir,subdir)
      box_message,'Using parent directory: ' + outdir
      week=''
   endcase
   else: begin
      box_message,'Need to supply either week (YYYY_WW) or file list (IFILES=filelist)'
      return
   endcase
endcase   

outfile=concat_dir(outdir,prefix+froot+'_'+flagstring+ mk_ssc_version)

if keyword_set(filename_only) then begin
   box_message,'Just returing filename in OUTFILE...'
   return
endif   


states=(['DISABLED','ENABLED'])
status=['Output File Name: ' + outfile,      $
  'Processing Options:' ,                    $
  '   Scattered Light:     ' + states(scatter), $
  '   Vignette Correction :' + states(vignette),$
  '   SXT_Clean (despike) :' + states(clean),   $
  '   SXT_XSAA  (destreak):' + states(destreak),   $
  '   Normal->Cts/Sec/Pix: ' +       states(normalize), $
  '   2nd Order Leak Corr: ' +       states(soleak) , $
  '   Synthetic SFC Corr:  ' + states(synsfc) ]

if noleak then begin
   box_message,'LEAK SUBTRACTION INHIBITED!',btext=txt,/quiet
   status=[txt,status]
endif 

box_message,status

rd_xda,files,-1,roadmap,xdata  
; 'roadmap' is really 'index' records since
; CCD Te check was added on 19-nov-2003
lweek=week

; -------- make LONG/SHORT pariings ----------
if loud then box_message,'Calling image-pairing routine...'
case 1 of 
   keyword_set(triplet): begin
      mk_sff_pair,roadmap,ssl,ssm,sss,/triplet,delta=delta, $
         saa_state=saa_state ,count=count,_extra=_extra,maxdt=660  ;triplets
   endcase
   else: begin
      mk_sff_pair,roadmap,ssl,ssm,sss,delta=delta, $
	 saa_state=saa_state,count=count,_extra=_extra,maxdt=660  ;doublets
   endcase
endcase

nlong=n_elements(ssl)
if n_elements(ss) ne 0 then nlong=ss
if count eq 0 then nlong=0

if loud then box_message,'Number of useful LONGS found: ' + strtrim(nlong,2)

for i=0,nlong(0) -1 do begin
  delvarx, data, fillmap
  if triplet then begin
      case 1 of					;LWA 1/24/04
         ssm(i) ne -1 and sss(i) ne -1 : ssx=[ssl(i),ssm(i),sss(i)]
         ssm(i) ne -1 and sss(i) eq -1 : ssx=[ssl(i),ssm(i)]
         ssm(i) eq -1                  : ssx=ssl(i)
      endcase
      rd_sda,files,ssx,index,data
  endif else begin
      case 1 of                                 ;LWA 1/24/04
         sss(i) ne -1                  : ssx=[ssl(i),sss(i)]
         sss(i) eq -1                  : ssx=ssl(i)
      endcase
      rd_sda,files,ssx,index,data
  endelse

     ; ---- Print out the images chosen for composites ---- 
     if loud then begin
        if keyword_set(triplet) then begin
          case 1 of 
            ssm(i) eq -1 and sss(i) eq -1 : begin 
	      box_message,['SST "Triplet" comprises:', $
	      get_info(roadmap(ssl(i)),/non)]
            end
            ssm(i) gt -1 and sss(i) eq -1 : begin
              box_message,['SST "Triplet" comprises:', $
              get_info(roadmap(ssl(i)),/non), $
              get_info(roadmap(ssm(i)),/non)] 
            end
	    ssm(i) gt -1 and sss(i) gt -1 : begin
              box_message,['SST "Triplet" comprises:', $
              get_info(roadmap(ssl(i)),/non), $
              get_info(roadmap(ssm(i)),/non), $
              get_info(roadmap(sss(i)),/non)]
              end
          endcase   
        endif else begin		 
          case 1 of
            sss(i) eq -1 : begin
              box_message,['SSC "Doublet" comprises:', $
              get_info(roadmap(ssl(i)),/non)]
            end
            sss(i) gt -1 : begin
              box_message,['SSC "Doublet" comprises:', $
              get_info(roadmap(ssl(i)),/non), $
              get_info(roadmap(sss(i)),/non)]
            end
          endcase
       endelse
     endif  

  if n_elements(ssx) gt 1 then begin    ;LWA 1/24/04
;    ---- Call compositing routine (calls sxt_prep internally) ---

; /Uncert choice for compositing, next 7 lines; 11-dec-01, DMcK
    if keyword_set(uncert) then sxt_sff_composite,index,data,cindex,cdata,$
        vignette=vignette, sxt_cleanx=clean, destreak=destreak, noleak=noleak, $
           fillmap=fillmap, unc_out=cunc ;, second_order_leak=soleak
    if not(keyword_set(uncert)) then sxt_sff_composite,index,data,cindex,cdata,$
        vignette=vignette, sxt_cleanx=clean, destreak=destreak, noleak=noleak, $
           fillmap=fillmap ;, second_order_leak=soleak
  endif else begin

;      ----- No compositing needed - just call sxt_prep ---------
       sxt_prep, index(0),data(*,*,0), cindex, cdata, cunc, $
          /float,/dc_orbit_correct,vignette=vignette, $
          sxt_cleanx=clean, destreak=destreak, noleak=noleak ;, $
          ;second_order_leak=soleak ; comm. out 19-nov-2003, LWA restored 1/2/04
       cunc=sxt_dn_unc(cindex,cdata,temporary(cunc))
       fillmap=bytarr(data_chk(data,/nx),data_chk(data,/ny))
   endelse

;  --------------- scatter correction ------------------   
   if scatter then begin 
      box_message,'Scatter; deconvolving using sxt_decon...'
      sxt_decon,cindex, cdata, scatindex, scatdata    ; << data array
      cdata=temporary(scatdata)
      sxt_decon,cindex,cunc,scatindunc,scatatunc      ; << uncertainty array
      cindex=scatindex
      cunc=temporary(scatatunc)
   endif

;  ----------- normalization ---------------
   case 1 of
      normalize: begin                               ; -> counts/sec/pixel
         box_message,'Normalizing to CTS/Sec/Pixl'
	 for j=0,data_chk(cdata,/nimage)-1 do $
	      cdata(0,0,j) = 1.e3*cdata(*,*,j)/gt_expdur(cindex(j))
	 his_index, cindex, -1, 'expdur', 1000.    ; history
         his_index, cindex, -1, 'q_exp_norm', 2    ; algorithm
         if keyword_set(uncert) then begin
	    for j=0,data_chk(cdata,/nimage)-1 do $
	      cunc(0,0,j) = 1.e3*cunc(*,*,j)/gt_expdur(cindex(j))
         endif   
      endcase

      deka: begin
         box_message,'DEKA-Normalizing'       
         cdata=deka_norm(cindex, cdata, deka=cindex)       
         cunc=deka_norm(cindex, cunc, deka=cindex)       

      endcase
      else: box_messge,'Not Normalizing data'
   endcase
;  ----------- Finally do 2nd order leak ------
   if soleak then begin
      raw=tim2dset(index,cindex)
      cdata=sxt_deleak(cindex,cdata,xdata=data(*,*,raw))
   endif
;  ----------- Save the SSC and go on to next case ------
   sav_sda,outfile,cindex,cdata,append=(i ne 0)

; Finally, save the uncert; 11-dec-01, DMcK
   if keyword_set(uncert) then $
      sav_sda,outfile+'_unc',cindex,cunc,append=(i ne 0)

endfor  
if debug then stop

return
end

